from Products.CMFCore.utils import getToolByName
from Products.CMFFormController.FormAction import FormActionKey
from Products.Archetypes.Extensions.utils import installTypes
from Products.Archetypes.Extensions.utils import install_subskin
from Products.Archetypes.public import listTypes

from StringIO import StringIO

from Products.CMFDynamicViewFTI.migrate import migrateFTIs

from Products.ATGoogleMaps.config import *

def install(self):
    """Install ATGoogleMaps: Install content types, skin layer, install the
    stylesheet, set up global properties, enable the portal factory and
    set up form controller actions for the widget actions
    """

    out = StringIO()

    print >> out, "Installing ATGoogleMaps"

    # Install types
    classes = listTypes(PROJECTNAME)
    installTypes(self, out,
                 classes,
                 PROJECTNAME)
    print >> out, "Installed types"

    # Install skin
    install_subskin(self, out, GLOBALS)
    print >> out, "Installed skin"

    # Migrate FTI, to make sure we get the necessary infrastructure for the
    # 'display' menu to work.
    migrated = migrateFTIs(self, product=PROJECTNAME)
    print >>out, "Switched to DynamicViewFTI: %s" % ', '.join(migrated)

    # Enable portal_factory
    factory = getToolByName(self, 'portal_factory')
    types = factory.getFactoryTypes().keys()
    for add_type in ('GMap','GMarker','GGPXPolyline','GImageMarker' ):
        if add_type not in types:
            print >> out, "add:%s" % add_type
            types.append(add_type)
            factory.manage_setPortalFactoryTypes(listOfTypeIds = types)

    print >> out, "Added ATGoogleMaps contents to portal_factory"

    portal_props = getToolByName(self, 'portal_properties')
    siteProperties = getattr(portal_props, 'site_properties')
    navtreeProperties = getattr(portal_props, 'navtree_properties')

    # Add to default_page_types
    defaultPageTypes = list(siteProperties.getProperty('default_page_types'))
    item = 'GMap'
    if item not in defaultPageTypes:
        print >> out, "%s append -> dsefaultPageTypes" % item
        defaultPageTypes.append( item )
    siteProperties.manage_changeProperties(default_page_types = defaultPageTypes)

    # add GImageMaker to typesUseViewActionInListings
    view_action_types = list(siteProperties.getProperty('typesUseViewActionInListings')) 
    if (not 'GImageMarker' in view_action_types):
        view_action_types.append('GImageMarker')
        print >> out, "GImageMarker append -> typesUseViewActionInListings"
        siteProperties._updateProperty('typesUseViewActionInListings',view_action_types)

    # Add property sheet
    if not hasattr(portal_props, PROPERTY_SHEET):
        portal_props.addPropertySheet(PROPERTY_SHEET, 'ATGoogleMaps properties')
        print >> out, "add ATGoogleMaps properties -> property sheet"
    props = getattr(portal_props, PROPERTY_SHEET)
    if not hasattr(props, 'api_key'):
        props._setProperty('api_key', '', 'string')

    # Add to parentMetaTypesNotToQuery
    #parentMetaTypesNotToQuery = list(navtreeProperties.getProperty('parentMetaTypesNotToQuery'))
    #if 'GMap' not in parentMetaTypesNotToQuery:
    #    parentMetaTypesNotToQuery.append('GMap')
    #navtreeProperties.manage_changeProperties(parentMetaTypesNotToQuery = parentMetaTypesNotToQuery)

    # Add All ATGoogleMap's objects to kupu's linkable and media
    # types
    kupuTool = getToolByName(self, 'kupu_library_tool')
    linkable = list(kupuTool.getPortalTypesForResourceType('linkable'))
    mediaobject = list(kupuTool.getPortalTypesForResourceType('mediaobject'))
    if 'GMarker' not in linkable:
        linkable.append('GMarker')
    if 'GImageMarker' not in linkable:
        linkable.append('GImageMarker')
    if 'GGPXPolyline' not in linkable:
        linkable.append('GGPXPolyline')
    if 'GMap' not in linkable:
        linkable.append('GMap')
    # kupu_library_tool has an idiotic interface, basically written purely to
    # work with its configuration page. :-(
    kupuTool.updateResourceTypes(({'resource_type' : 'linkable',
                                   'old_type'      : 'linkable',
                                   'portal_types'  :  linkable},
                                  {'resource_type' : 'mediaobject',
                                   'old_type'      : 'mediaobject',
                                   'portal_types'  :  mediaobject},))

    install_configlet(self, out)

    return out.getvalue()

def install_configlet(self, out):
    try:
	portal_conf = getToolByName(self, 'portal_controlpanel')
    except AttributeError:
	print >>out, "Configlet could not be installed"
    try:
	portal_conf.registerConfiglet('%sSettings' % PROJECTNAME,
				      '%s Settings' % PROJECTNAME,
				      'string:${portal_url}/prefs_gmap_form',
				      '',
				      'Manage Portal',
				      'Products',
				      1,
				      PROJECTNAME,
				      'gmap_icon.gif',
				      '',
				      None,
				      )
    except KeyError:
        pass # Get KeyError when registering duplicate configlet.

def uninstall(self, reinstall):
    out = StringIO()

    if not reinstall:
        print >> out, "Uninstalling ATGoogleMaps"

        portal_props = getToolByName(self, 'portal_properties')
        siteProperties = getattr(portal_props, 'site_properties')
        navtreeProperties = getattr(portal_props, 'navtree_properties')

        # Remove to default_page_types
        defaultPageTypes = list(siteProperties.getProperty('default_page_types'))
        item = 'GMap'
        if item not in defaultPageTypes:
            print >> out, "%s remove <- dsefaultPageTypes" % item
            defaultPageTypes.remove( item )
        siteProperties.manage_changeProperties(default_page_types = defaultPageTypes)

        # remove GImageMarker from typesUseViewActionInListings
        view_action_types = list(siteProperties.getProperty('typesUseViewActionInListings'))
        if 'GImageMarker' in view_action_types:
            print >> out, "GImageMarker remove <- typesUseViewActionInListings"
            view_action_types.remove('GImageMarker')
        view_action_types = tuple(view_action_types)
        siteProperties._updateProperty('typesUseViewActionInListings',view_action_types)

        # Remove property sheet
        if hasattr(portal_props, PROPERTY_SHEET):
            portal_props.manage_delObjects(PROPERTY_SHEET)
            print >> out, "del ATGoogleMaps properties <- property sheet"

    # remove the configlet from the portal control panel
    configTool = getToolByName(self, 'portal_controlpanel', None)
    if configTool:
        configTool.unregisterConfiglet('%sSettings' % PROJECTNAME)
        print >> out, 'Removed ATGoogleMaps configlet'

    return out.getvalue()
