"""A Google Marker type for Images"""

__author__  = '''774RR <774RR@hogehogehogejohn.doe>'''
__docformat__ = 'plaintext'


from AccessControl import ClassSecurityInfo

from Products.ATContentTypes.content.image import ATImageSchema
from Products.ATContentTypes.content.image import ATImage
from Products.ATContentTypes.content.base import updateActions
from Products.ATContentTypes.content.base import ATCTFileContent
from Products.ATContentTypes.lib.imagetransform import ATCTImageTransform
from Products.ATContentTypes.content.schemata import finalizeATCTSchema

from Products.Archetypes.public import Schema
from Products.Archetypes.public import ImageField
from Products.Archetypes.public import ImageWidget
from Products.Archetypes.public import PrimaryFieldMarshaller
try:
    from Products.FileSystemStorage.FileSystemStorage import FileSystemStorage
except:
    raise DependancyError('If you want to use FileSystemStorage you need to install the FileSystemStorage Product')

from Products.ATContentTypes.configuration import zconf
from Products.validation import V_REQUIRED

try:
  from Products.LinguaPlone.public import *
except ImportError:
  # No multilingual support
  from Products.Archetypes.public import *

from Products.ATGoogleMaps.config import *
from Products.ATGoogleMaps.field import LatLngField
from Products.ATGoogleMaps.widget import LatLngWidget
from Products.ATGoogleMaps.lib.gpsdata import *

GImageMarkerSchema = ATImageSchema.copy() + Schema((
),
)

Fullschema = GImageMarkerSchema.copy()
del Fullschema['image']
GImageMarkerSchema = Fullschema + Schema((
    ImageField('image',
        required=True,
        primary=True,
        languageIndependent=True,
        storage = FileSystemStorage(),
        swallowResizeExceptions = zconf.swallowImageResizeExceptions.enable,
        pil_quality = zconf.pil_config.quality,
        pil_resize_algo = zconf.pil_config.resize_algo,
        max_size = zconf.ATImage.max_image_dimension,
        sizes= {'large'   : (768, 768),
                'preview' : (400, 400),
                'mini'    : (200, 200),
                'thumb'   : (128, 128),
                'tile'    :  (64, 64),
                'icon'    :  (32, 32),
                'listing' :  (16, 16),
               },
        validators = (('isNonEmptyFile', V_REQUIRED),
                      ('checkImageMaxSize', V_REQUIRED)),
        widget = ImageWidget(
             #description = "Select the image to be added by clicking the 'Browse' button.",
             #description_msgid = "help_image",
             description = "",
             label= "Image",
             label_msgid = "label_image",
             i18n_domain = "plone",
             macro='fss_file_widget',
             show_content_type = False,
        )
    ),

    TextField('text',
        searchable=True,
        validators=('isTidyHtmlWithCleanup',),
        default_content_type=zconf.ATDocument.default_content_type,
        default_output_type='text/x-html-safe',
        allowable_content_types=zconf.ATDocument.allowed_content_types,
        widget=RichWidget(
            rows=10,
            label="Text",
            label_msgid="label_text",
            description_msgid="help_text",
            i18n_domain="googlemaps",
        ),
    ),

    # hidden data start
    LatLngField('point',
        default_method='getGMapCenter',
        widget=LatLngWidget(
            label='Point',
            label_msgid='label_point',
            description='Point of this marker.',
            description_msgid='help_point',
            i18n_domain='googlemaps',
            size=12,
            visible = {'edit':'invisible', 'view':'visible'},
        )
    ),

    StringField('detum',
        widget=StringWidget(
            label='Detum',
            label_msgid='label_detum',
            description_msgid='help_detum',
            i18n_domain='googlemaps',
            visible = {'edit':'invisible', 'view':'visible'},
        )
    ),

    StringField('gpsLatitude',
        widget=StringWidget(
            label='GPS Latitude',
            label_msgid='label_gpsLatitude',
            description_msgid='help_gpsLatitude',
            i18n_domain='googlemaps',
            visible = {'edit':'invisible', 'view':'visible'},
        )
    ),

    StringField('gpsLongitude',
        widget=StringWidget(
            label='GPS Longitude',
            label_msgid='label_gpsLongitude',
            description_msgid='help_gpsLongitude',
            i18n_domain='googlemaps',
            visible = {'edit':'invisible', 'view':'visible'},
        )
    ),

    StringField('gpsDatetime',
        widget=StringWidget(
            label='GPS Datetime',
            label_msgid='label_gpsDatetime',
            description_msgid='help_gpsDatetime',
            i18n_domain='googlemaps',
            visible = {'edit':'invisible', 'view':'visible'},
        )
    ),
    # hidden data end

    StringField('address',
        widget=IntegerWidget(
            label='Address',
            label_msgid='label_address',
            description_msgid='help_address',
            i18n_domain='googlemaps',
            size=30,
        )
    ),
    
    StringField('phone',
        widget=IntegerWidget(
            label='Phone',
            label_msgid='label_phone',
            description_msgid='help_phone',
            i18n_domain='googlemaps',
            size=30,
        )
    ),
    
    StringField('url',
        widget=IntegerWidget(
            label='URL',
            label_msgid='label_url',
            description_msgid='help_url',
            i18n_domain='googlemaps',
            size=60,
        )
    ),
    
    StringField('tab1',
        widget=IntegerWidget(
            label='Tab #1 title',
            label_msgid='label_tab1',
            description_msgid='help_tab1',
            i18n_domain='googlemaps',
            size=20,
        ),
        schemata='googlemaps_tabbed',
    ),
    
    StringField('tab2',
        widget=IntegerWidget(
            label='Tab #2 title',
            label_msgid='label_tab2',
            description_msgid='help_tab2',
            i18n_domain='googlemaps',
            size=20,
        ),
        schemata='googlemaps_tabbed',
    ),

    TextField('detail',
        searchable=True,
        validators=('isTidyHtmlWithCleanup',),
        default_content_type=zconf.ATDocument.default_content_type,
        default_output_type='text/x-html-safe',
        allowable_content_types=zconf.ATDocument.allowed_content_types,
        widget=RichWidget(
            rows=10,
            label="Detail",
            label_msgid="label_detail",
            description_msgid="help_detail",
            i18n_domain="googlemaps",
        ),
        schemata='googlemaps_tabbed',

    ),
), marshall=PrimaryFieldMarshaller(),
)

#Finalize schema definition
finalizeATCTSchema(GImageMarkerSchema)

class GImageMarker(ATImage,GPSDataConv):
    """Google Image Marker"""

    archetype_name  = 'GImageMarker'

    schema          = GImageMarkerSchema
    content_icon    = 'gimagemarker_icon.gif'
    portal_type     = 'GImageMarker'
    meta_type       = 'GImageMarker'
    default_view    = 'gimagemarker_view'
    immediate_view  = 'gimagemarker_view'
    suppl_views     = ()
    typeDescription = 'Google Image marker document'
    typeDescMsgId   = 'description_edit_gimagemarker'

    __implements__ = (getattr(ATImage,'__implements__',()),)


    global_allow    = 0
    security = ClassSecurityInfo()

    # Make sure we get title-to-id generation when an object is created
#    _at_rename_after_creation = True

    security.declarePrivate('setImage')
    def setImage(self, value, refresh_exif=True, **kwargs):
        exif_data = self.getEXIF(value, refresh=refresh_exif)
        # get GPS Info & set Center LatLng
        self.getEXIFGPSInfo( exif_data )
        self._setATCTFileContent(value, **kwargs)

    security.declarePrivate('setPoint')
    def setPoint(self, value, cb=None, **kw ) :
        """set the Point"""
        if cb or self.isKwStatus( kw, '_initialize_') :
            f=self.Schema()['point']
            log( "setPoint value=%s" % value )
            return f.set(self, value, **kw)
        return None

    security.declarePrivate('setDetum')
    def setDetum(self, value, cb=None, **kw ) :
        """set the Point"""
        if cb or self.isKwStatus( kw, '_initialize_') :
            f=self.Schema()['detum']
            log( "setDetum value=%s" % value )
            return f.set(self, value, **kw)
        return None

    security.declarePrivate('setGpsLatitude')
    def setGpsLatitude(self, value, cb=None, **kw ) :
        """set the GpsLatitude"""
        if cb or self.isKwStatus( kw, '_initialize_') :
            f=self.Schema()['gpsLatitude']
            log( "setGpsLatitude value=%s" % value )
            return f.set(self, value, **kw)
        return None

    security.declarePrivate('setGpsLongitude')
    def setGpsLongitude(self, value, cb=None, **kw ) :
        """set the GpsLongitude"""
        if cb or self.isKwStatus( kw, '_initialize_') :
            f=self.Schema()['gpsLongitude']
            log( "setGpsLongitude value=%s" % value )
            return f.set(self, value, **kw)
        
        return None

    security.declarePrivate('setGpsDatetime')
    def setGpsDatetime(self, value, cb=None, **kw ) :
        """set the GpsLongtude"""
        if cb or self.isKwStatus( kw, '_initialize_') :
            f=self.Schema()['gpsDatetime']
            log( "setGpsDatetime value=%s" % value )
            return f.set(self, value, **kw)
        return None

    # default point value from parent GMap center
    def getGMapCenter(self):
        latlng = {'latitude': '0.0', 'longitude': '0.0'}
        try:
            latlng = self.aq_parent.getCenter()
        except:
            pass
        return latlng

    def getEXIFGPSInfo( self, exif ):

        rtn = None

        # Detum
        detum = exif.get('GPS GPSMapDatum', 'No Data' )
        self.setDetum( detum, cb=True )
        
        # Latitude & altitude
        latlng = {}
        lat = exif.get('GPS GPSLatitude', None )
        lon = exif.get('GPS GPSLongitude', None )
        if lat and lon :
            deglat = self.dms2deg( lat )
            deglon = self.dms2deg( lon )
            latlng = self.conv2WGSdetumlatlng( deglat, deglon, detum )
            self.setPoint( latlng, cb=True )
            latr = exif.get('GPS GPSLatitudeRef', 'N' )
            lat = self.dms2str( lat )
            strlat = "%s  %s %s'%s\"" % ( latr, lat[0], lat[1], lat[2] )
            self.setGpsLatitude( strlat, cb=True )
            lonr = exif.get('GPS GPSLongitudeRef', 'E' )
            lon = self.dms2str( lon )
            strlon = "%s  %s %s'%s\"" % ( lonr, lon[0], lon[1], lon[2] )
            self.setGpsLongitude( strlon, cb=True )
        else :
            kw = { '_initialize_': True }
            self.setPoint( {}, **kw )
            self.setGpsLatitude( 'No Data', **kw )
            self.setGpsLongitude( 'No Data', **kw )
        
        #GPS DateTime
        dates = exif.get('GPS GPSDateStamp', None )
        times = exif.get('GPS GPSTimeStamp', None )
        t = ''
        if times and dates :
            dt = "%s %s" % (dates, ":".join(eval(times)))
            t = time.strptime( dt, "%Y:%m:%d %H:%M:%S")
            t = datetime( year=t[0],month=t[1],day=t[2], hour=t[3], minute=t[4], second=t[5] )
            self.setGpsDatetime( "%s" % t, cb=True )
        elif dates :
            t = time.strptime( dates, "%Y:%m:%d")
            t = datetime( year=t[0],month=t[1],day=t[2] )
            self.setGpsDatetime( "%s" % t, cb=True )
        else :
            kw = { '_initialize_': True }
            self.setGpsDatetime( 'No Data', **kw )

    security.declarePrivate('isKwStatus')
    def isKwStatus( self, kw, key ):
        if kw.has_key( key ) :
            if kw[key] :
                return True
        return False

    # default point value from parent GMap center
    def getGMapCenter(self):
        latlng = {'latitude': '0.0', 'longitude': '0.0'}
        try:
            latlng = self.aq_parent.getCenter()
        except:
            pass
        return latlng

registerType(GImageMarker,PROJECTNAME)
