from datetime import tzinfo, timedelta, datetime
import time
import math

class UTC(tzinfo):
    """UTC"""
    def utcoffset(self, dt):
        return timedelta(0)

    def tzname(self, dt):
        return "UTC"

    def dst(self, dt):
        return timedelta(0)

class JST(tzinfo):
    """JST"""
    def utcoffset(self, dt):
        return timedelta(hours=9)

    def tzname(self, dt):
        return "JST"

    def dst(self, dt):
        return timedelta(hours=9)

class GPSDataConv :

    def convUTCstr2JST( self, s, fmt ) :
        # GPXにある時刻はUTCと決めつけて処理
        # UTC -> JST 変換はこの辺を参照してtestしまくった...orz
        # http://www.python.jp/doc/2.4/lib/datetime-datetime.html
        # http://www.python.jp/doc/2.4/lib/datetime-tzinfo.html
        utc = UTC()
        jst = JST()
        
        try :
            t = time.strptime( s, fmt )
            ut = datetime( year=t[0],month=t[1],day=t[2],
                           hour=t[3], minute=t[4], second=t[5], tzinfo=utc )
        except :
            return None

        return ut.astimezone(jst)

    def chgDegreeToRadian( self, data ) :
        return (data / 360 * 2 * math.pi)

    def getXYDisplacement( self, lat1, lon1, lat2, lon2 ) :
        # http://wadati.blog10.fc2.com/blog-entry-345.html

        A = 6378137

        if isinstance( lat1,(str, unicode) ) : lat1 = eval( lat1 )
        lat1 = self.chgDegreeToRadian( lat1 )
        if isinstance( lat2,(str, unicode) ) : lat2 = eval( lat2 )
        lat2 = self.chgDegreeToRadian( lat2 )
        if isinstance( lon1,(str, unicode) ) : lon1 = eval( lon1 )
        lon1 = self.chgDegreeToRadian( lon1 )
        if isinstance( lon2,(str, unicode) ) : lon2 = eval( lon2 )
        lon2 = self.chgDegreeToRadian( lon2 )

        lat = lat2 - lat1
        lon = lon2 - lon1
        lon_x = A * lon * math.cos(lat1)
        lat_y = A * lat

        return [lon_x,lat_y]
        
    def getDistance( self, lat1, lon1, lat2, lon2 ) :
        #----------------------------------------------------------------------
        # [memo]         ____________________________________
        #               /               2                 2
        #     L(2D) = ~V  x_Displacement  + y_Displacement
        #----------------------------------------------------------------------

        xy_disp = self.getXYDisplacement( lat1, lon1, lat2, lon2 )
        return math.hypot( xy_disp[0], xy_disp[1] )

    def getDistance3D( self, lat1, lon1, lat2, lon2, ele_diff ) :
    
        dist_2D = self.getDistance( lat1, lon1, lat2, lon2 )
        if isinstance(ele_diff,str) or isinstance(ele_diff,unicode) :
            ele_diff = eval( ele_diff )
        # c**2 = a**2 + b**2
        return math.hypot( dist_2D, ele_diff )

    def getBearings( self, lat1, lon1, lat2, lon2 ) :
        #----------------------------------------------------------------------
        # [memo]
        #   Bearings WEST = 0
        #             -1
        #   Bearings = tan ( y_Displacement / x_Displacement ) * 180 / PI
        #----------------------------------------------------------------------

        xy_disp = self.getXYDisplacement( lat1, lon1, lat2, lon2 )

        a = self.getAngle( xy_disp[0], xy_disp[1] )
        if a < 0 : a = 360 + a
        a = math.fmod( (360 - a) + 90, 360 )
        return a

    def getAngle( self, x , y ) :
        a = 0
        if x != 0 or y != 0:
            at = math.atan2( y ,x )
            a  = at * 180 / math.pi
        return a

    def getSpeedMPerM ( self, dist, seconds ) :
        if seconds != 0 :
            return dist / seconds
        return 0

    def getSpeedKmPerH ( self, dist, seconds ) :
        V = self.getSpeedMPerM( dist, seconds )
        return (V * 60 * 60 /1000)

    def cngSpeedKmPerH2MPerM ( self, speed ) :
        return ( (speed * 1000) / (60*60) )

    def getAcceleration ( self, src_speed, dst_speed, seconds ) :
        src_v = self.cngSpeedKmPerH2MPerM( src_speed )
        dst_v = self.cngSpeedKmPerH2MPerM( dst_speed )
        if seconds != 0 :
            return( (dst_v - src_v) / seconds )
        return 0

    def dms2deg( self, dms ) :

        if not dms :
            return None
        
        rtn = 0.0
        dms = eval(dms)
        c = [1,60,3600]
        for i in range(0,len(dms)) :
            sp = dms[i].split('/')
            for j in range( 0,len(sp) ) :
                sp[j] = float(eval(sp[j]))
            if len(sp) == 2 :
                dms[i] = float( sp[0] / sp[1] )
            else :
                dms[i] = float( sp[0] )
            dms[i] = dms[i] / float(c[i])
            rtn += dms[i]
        return rtn

    def dms2str( self, dms ) :

        if not dms :
            return None

        dms = eval(dms)
        for i in range(0,len(dms)) :
            sp = dms[i].split('/')
            for j in range( 0,len(sp) ) :
                sp[j] = float(eval(sp[j]))
            if len(sp) == 2 :
                dms[i] = "%.4f" % float(sp[0]/sp[1])
            else :
                dms[i] = str(int(sp[0]))
        return dms

    def conv2WGSdetumlatlng( self, lat, lon, detum ):
        latlng = {'latitude': '', 'longitude': ''}
        
        if not lat and not lon :
            return latlng
        
        if detum == 'WGS-84' :
            latlng['latitude'] = str(lat)
            latlng['longitude'] = str(lon)
            return latlng
        elif detum == 'Tokyo' :
            wlat = lat - 0.00010695*lat + 0.000017464*lon + 0.0046017
            wlon = lon - 0.000046038*lat - 0.000083043*lon + 0.010040
            latlng['latitude'] = str(wlat)
            latlng['longitude'] = str(wlon)
            return latlng
        return latlng

