from gpsdata import *
from xml.dom.ext.reader.Sax import FromXmlStream
from datetime import timedelta

MAX_BEARINGS = 360.0
MAX_DIST     = 999999999.0

###############################################################################
#
#  *.gpx file Analyze Class
#    ・<trkseg>は無視でfile内にある<trkpt>をいっしょくたに扱ってたりする
#    ・<trk>が複数出てもいっしょくた(多分)
#    ・<wpt>は・・・どうしよーか
#
###############################################################################
class GPXAnalyze( GPSDataConv ):

    def __init__(self, debug=0):
        self.gpx_data = {}
        self.debug = debug
        self.item_list = {'trk':['lon', 'lat', 'ele', 'time'],\
                          'wpt':['lon', 'lat', 'ele', 'time', 'cmt']}

    def log( self, logstr ) :
        if self.debug : print logstr

    def makeItemListStr( self, list ) :
        s = "{"
        for i in list :
            s += "'%s':''," % i
        s += "}"
        return s

    def initTrkItems( self, name ) :
        
        s = self.makeItemListStr( self.item_list[name] )
        self.gpx_data[name].append(eval(s))

    def setTrkItemList( self, name, value ):
        
        item = value
        if name == 'time' :
            item = self.convUTCstr2JST( value, "%Y-%m-%dT%H:%M:%SZ" )
        now_trkpt = len(self.gpx_data['trk']) - 1
        self.gpx_data['trk'][now_trkpt][name] = item
        self.log( "%s(%d)=%s" % (name,len(self.gpx_data['trk']),item) )

    def setTrkItemValue( self, name, value ):
        self.gpx_data[name] = value
        self.log( "%s=%s\n" % (name, value) )

    def getNodeAttr( self, node ):
        name  = node.nodeName
        attrs = node.attributes
        if attrs != None and len(attrs) > 0:
            for a in attrs:
                if name == 'trkpt' and (a.name == 'lat' or a.name == 'lon'):
                    self.setTrkItemList( a.name, a.value )
                if name == 'gpx' and a.name == 'creator':
                    self.setTrkItemValue( a.name, a.value )

    def nodeInspect( self, node, pname='' ):
        name = node.nodeName
        value = node.nodeValue

        if name == 'gpx':
            self.getNodeAttr( node )
        elif name == 'trk' :
            self.gpx_data['trk'] = []
        elif name == 'trkpt' :
            self.initTrkItems( 'trk' )

        if pname == "name" :
            self.setTrkItemValue( pname, value )
        elif pname == "trkseg":
            self.getNodeAttr( node )
        elif pname in ('ele','time') :
            self.setTrkItemList( pname, value )

        children = node.childNodes
        for c in children:
          self.nodeInspect( c, name )

    def checkTrkData( self, trk_data ) :
    
        # time check (all none ?)
        timef = 0
        for j in trk_data :
            if j['time'] != '' :
                timef = 1
                break
        
        if timef == 0 :
            for j in trk_data :
                del( j['time'] )

        elef  = 0
        for j in trk_data :
            if j['ele'] != '' :
                elef = 1
                break
        if elef == 0 :
            for j in trk_data :
                del( j['ele'] )

        dellist = []
        for i in range( len(trk_data) ) :
            if   trk_data[i]['lon'] == '' or eval(trk_data[i]['lon']) == 0 :
                dellist.append(i)
            elif trk_data[i]['lat'] == '' or eval(trk_data[i]['lat']) == 0 :
                dellist.append(i)
            elif trk_data[i].has_key('time') :
                if trk_data[i]['time'] == ''  or None :
                    dellist.append(i)
            elif trk_data[i].has_key('ele') :
                if trk_data[i]['ele']  == '' :
                    dellist.append(i)
        
        dellist.reverse()
        for i in dellist :
            del( trk_data[i] )
        if len(dellist) : self.log( "checkTrkData() : illegal data found" )

    def getMaxMinAvgTotal( self, list, target ) :
        total = timedelta(0)
        rtn = {}
        l_list = []
        is_time = 0

        #print type(list[0])
        if type( list[0][target] ) == type(total) :
            is_time = 1
        else :
            total = 0.0

        for i in list :
            l_list.append( i[target] )
            total += i[target]
        
        if is_time == 1 :
            rtn['min'] = list[0]['time']
            rtn['max'] = list[-1]['time']
        else :
            l_list.sort()
            l_list.reverse()
            rtn['max']   = l_list[0]
            rtn['min']   = l_list[len(l_list)-1]
            avg = total / len(l_list)
            if target in ('speed','speed_3D') :
                try :
                    d = self.gpx_data['dist_data']['total']
                    t = self.gpx_data['moving_time']['total']
                    avg = self.getSpeedKmPerH( d, t.seconds )
                except :
                    try :
                        d = self.gpx_data['dist_data']['total']
                        t = self.gpx_data['time_diff']['total']
                        avg = self.getSpeedKmPerH( d, t.seconds )
                    except :
                        pass
            rtn['avg'] = avg
            
        rtn['total'] = total

        return rtn

    def reviseAbnormalPoints( self, trk_list, v_diff=50 ) :
        
        for i in trk_list :
            i['rsf'] = 0

        #######################################################################
        #search for the point where the difference of velocity is abnormal.
        #   abnormal points = difference >= v_diff
        #######################################################################
        # Does necessary data exist?
        try :
            t = trk_list[0]['dist']
            t = trk_list[0]['time_diff']
            t = trk_list[0]['speed']
        except :
            return

        data = []
        for i in trk_list :
            trk_idx = trk_list.index(i)
            data.append( { 'idx':trk_idx,
                           'dist':i['dist'],
                           'time_diff':i['time_diff'],
                           'speed':i['speed'] } )
        org_len = len(data)
        i = 2
        while i < len(data) :
            rsf = 0
            diff = abs(data[i]['speed'] - data[i-1]['speed'])
            dist = data[i]['dist']
            time_diff = data[i]['time_diff']
            speed = data[i]['speed']
            while (diff >= v_diff) and (i >= 2):
                dist += data[i-1]['dist']
                time_diff += data[i-1]['time_diff']
                idx = data[i-1]['idx']
                trk_list[idx]['rsf'] = 2
                del(data[i-1])
                speed = self.getSpeedKmPerH( dist, time_diff.seconds)
                diff = abs(speed - data[i-2]['speed'])
                i -= 1
                rsf = 1
            data[i]['dist'] = dist
            data[i]['time_diff'] = time_diff
            data[i]['speed'] = speed
            idx = data[i]['idx']
            trk_list[idx]['speed'] = speed
            trk_list[idx]['rsf'] = rsf
            i += 1

        i = 1
        while i < len(trk_list) :
            src_i = i - 1
            cnt = 0
            while (trk_list[i]['rsf'] == 2) and (i < len(trk_list)) :
                i += 1
                cnt += 1
            if cnt > 0 :
                x1 = 0
                y1 = trk_list[src_i]['speed']
                x2 = cnt+1
                y2 = trk_list[i]['speed']
                b = y1
                a = (y2 - b) / x2
                for x in range( x1+1, x2 ) :
                    y = a*x + b
                    trk_list[src_i+x]['speed'] = y
            i += 1
        
        rslt_len = len(data)
        print "org=%d,rslt=%d" % (org_len,rslt_len)
        self.gpx_data['speed_data'] = self.getMaxMinAvgTotal(trk_list, 'speed')
        self.log( "%s : %s" % ('speed', self.gpx_data['speed_data']) )


    def averageRunningData( self, trk_list, avg_dist=100 ) :
        
        if avg_dist == 0:
            return
        
        # check data alive
        try :
            t = trk_list[0]['dist']
            t = trk_list[0]['time_diff']
            t = trk_list[0]['speed']
        except :
            return
        
        avgd = float(avg_dist)
        # speed revise
        avgd_cnt = 0.0
        cnt = 0
        time_cnt = timedelta(0)
        for trk in trk_list :
            avgd_cnt += trk['dist']
            time_cnt += trk['time_diff']
            cnt += 1
            if avgd_cnt >= avgd :
                speed = self.getSpeedKmPerH( avgd_cnt, time_cnt.seconds )
                idx = trk_list.index(trk)
                self.log( "%04d - %04d : avg_sp = %f" % ((idx-(cnt-1)), idx, speed) )
                for i in range( idx-(cnt-1), idx+1) :
                    if i == 0 :
                        trk_list[i]['speed'] = 0
                    else :
                        trk_list[i]['speed'] = speed
                cnt = 0
                avgd_cnt = 0.0
                time_cnt = timedelta(0)

        self.gpx_data['speed_data'] = self.getMaxMinAvgTotal(trk_list, 'speed')
        self.log( "%s : %s" % ('speed', self.gpx_data['speed_data']) )


    def addRunningData( self, trk_list ) :

        trk_former = trk_list[0]
        for trk in trk_list :
            dist_3D = -1
            lat_1 = trk_former['lat']
            lon_1 = trk_former['lon']
            lat_2 = trk['lat']
            lon_2 = trk['lon']
            
            dist = self.getDistance( lat_1, lon_1, lat_2, lon_2)
            trk['dist'] = dist
            trk['angle'] = self.getBearings( lat_1, lon_1, lat_2, lon_2)
            try :
                if isinstance( trk['ele'],( str, unicode ) ) :
                    trk['ele'] = eval(trk['ele'])
                ele_diff = trk['ele'] - trk_former['ele']
                dist_3D = self.getDistance3D( lat_1, lon_1, lat_2, lon_2,
                                                   ele_diff )
                trk['dist_3D'] = dist_3D
            except :
                pass

            try :
                time_diff = trk['time'] - trk_former['time']
                trk['time_diff'] = time_diff
                trk['speed'] = self.getSpeedKmPerH( dist,
                                                    time_diff.seconds )
                idx = trk_list.index(trk)
                acc = 0
                if idx > 0 :
                    src_sp = trk_list[idx-1]['speed']
                    dst_sp = trk['speed']
                    acc = self.getAcceleration( src_sp, dst_sp,
                                                time_diff.seconds )
                trk['acc'] = acc
                moving_time = timedelta(0)
                if trk['speed'] > 1 :
                    moving_time = time_diff
                trk['moving_time'] = moving_time
            except :
                pass
            trk_former = trk
        
        #make max,min,avg,total data
        data = ['dist','time_diff','moving_time','ele','dist_3D','speed']
        for tgt in data :
            if trk_list[0].has_key( tgt ) :
                self.gpx_data[tgt+'_data'] = self.getMaxMinAvgTotal( trk_list, tgt )
                self.log( "%s : %s" % (tgt, self.gpx_data[tgt+'_data']) )

    def getBearingsDifference( self, a1, a2 ) :
        a_diff = abs(a1 - a2)
        if a_diff > 180 :
            a_diff = 360 - a_diff
        return a_diff

    def setThinOutData( self, idx, data ) :
        angle = 0
        dist = 0
        
        if idx > 0 and idx < len(data):
            angle = self.getBearings( data[idx-1]['lat'], data[idx-1]['lon'],
                                      data[idx]['lat'], data[idx]['lon'] )
            dist = self.getDistance( data[idx-1]['lat'], data[idx-1]['lon'],
                                     data[idx]['lat'], data[idx]['lon'] )
        try :
            data[idx]['angle'] = angle
            data[idx]['dist'] = dist
        except IndexError, s :
            self.log( "%s idx = %d" % (s, idx) )
            
    def updateThinOutDifference( self, idx, data ) :
        adiff = MAX_BEARINGS
        ddiff = MAX_DIST
        
        if (idx > 0) and (idx < (len(data) - 1)) :
            src_a = self.getBearingsDifference( data[idx-1]['angle'], 
                                                data[idx]['angle'] )
            del_a = self.getBearingsDifference( data[idx-1]['angle'], 
                                                data[idx+1]['angle'] )
            adiff = abs(src_a - del_a)
            
            src_d = data[idx]['dist'] + data[idx+1]['dist']
            del_d = self.getDistance( data[idx-1]['lat'],
                                      data[idx-1]['lon'],
                                      data[idx+1]['lat'],
                                      data[idx+1]['lon'] )
            ddiff = abs(src_d - del_d)
        try :
            data[idx]['adiff']  = adiff
            data[idx]['ddiff']  = ddiff
        except IndexError, s :
            self.log( "%s idx = %d" % (s, idx) )

    def thinOutTrackData( self, trk_data, rate ) :

        for i in trk_data :
            i['tof'] = False

        if rate == 100 : return
        
        max_len = len(trk_data)
        r = float(rate / 100.0)
        tgt_len = int( max_len * r )
        if tgt_len == 0 :
            self.log( "Thin Out Rate Err" )
            return
        
        mode = 'ddiff'
        
        self.log( "Thin Out(%d%%) : %d -> %d" % (rate, max_len, tgt_len ) )
        dellist = []
        cdata = []

        for i in trk_data :
            trk_idx = trk_data.index(i)
            i['tof'] = True
            cdata.append( { 'idx':trk_idx,
                            'lat':i['lat'],
                            'lon':i['lon'] } )

            self.setThinOutData( trk_idx, cdata )

        for i in cdata :
            idx = cdata.index(i)
            self.updateThinOutDifference( idx, cdata )

        while( len(cdata) >= tgt_len ) :
            min_value = MAX_DIST
            idx = 0
            for i in cdata :
                if min_value > i[mode] :
                    min_value = i[mode]
                    idx = cdata.index(i)
            self.log( "DEL DATA[%d]:A=%f D=%f" % ( cdata[idx]['idx'],
                            cdata[idx]['adiff'],cdata[idx]['ddiff'] ) )

            dellist.append( cdata[idx]['idx'] )
            del( cdata[idx] )

            self.setThinOutData( idx, cdata )
            self.updateThinOutDifference( idx, cdata )

        self.log( "POINT: ORG=%d DST=%d" % ( max_len, len(cdata)) )

        for i in cdata :
            trk_idx = i['idx']
            trk_data[trk_idx]['tof'] = False

    ###########################################################################
    #
    # public functions
    #
    ###########################################################################
    def makeLogData( self, io, rate=20, unit=100 ):

        try :
            io.seek(0)
            doc = FromXmlStream(io)
        except :
            self.log( "No GPX Track Log File" )
            return {}
        
        self.nodeInspect( doc )
        
        # data matching & check
        if self.gpx_data.has_key('trk') :
            self.checkTrkData( self.gpx_data['trk'] )
            if len( self.gpx_data['trk'] ) != 0 :
                self.addRunningData( self.gpx_data['trk'] )
                self.reviseAbnormalPoints( self.gpx_data['trk'] )
#                self.averageRunningData( self.gpx_data['trk'], unit )
                self.thinOutTrackData( self.gpx_data['trk'], rate  )
            else :
                self.gpx_data = {}
        else :
            self.gpx_data = {}

        return self.gpx_data

